package uk.co.his.test.simpleol5;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;
import java.util.UUID;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PostLoad;
import javax.persistence.Transient;
import javax.persistence.Version;

import uk.co.his.test.TestException;



@Entity
@NamedQueries({})
@Inheritance(strategy=InheritanceType.SINGLE_TABLE)
@DiscriminatorColumn
public class Container {

	@Id String id;
    
    @Version private int version;
    
    @Transient private boolean updatedIndices = false;

	public boolean isUpdatedIndices() {
		return updatedIndices;
	}

	public void setUpdatedIndices(boolean updatedIndices) {
		this.updatedIndices = updatedIndices;
	}

	public void setUUID() {
		id = UUID.randomUUID().toString();
	}
	
	 @Basic String timestamp;
	    
	    public String getTimestamp() {
			return timestamp;
		}

		public void setTimestamp(String timestamp) {
			this.timestamp = timestamp;
		}
		
	@SuppressWarnings("unused")
	@ManyToOne(fetch=FetchType.LAZY)
	private Singleton singleton;
	
	public void setSingleton(Singleton singleton) {
		this.singleton = singleton;
	}

	@OneToMany(cascade=CascadeType.ALL, mappedBy="container")
	@OrderBy("item_idx")
    java.util.List<Item>  items  = new ArrayList<Item>();
	
	public java.util.List<Item> getItems() {
		return items;
	}

	public  Item itemsAddNew(Item newObj, EntityManager em) {
	    newObj.setUUID();
	    newObj.setContainer(this);
	    em.persist(newObj);
	    items.add(newObj);

	    return newObj;
	}
	
	public  Item insertNewItemAtStart(Item newObj, EntityManager em) {
	    newObj.setUUID();
	    newObj.setContainer(this);
	    em.persist(newObj);
	    items.add(0, newObj);
	    return newObj;
	}
	
	
	private void z_updateIndices(int startIndex) {
		for (int i = startIndex; i<items.size();i++) {
			items.get(i).setItemIndex(i);
        }
	}
	
	public void touchItems() {
		items.size();
	}
	
    public void z_updateIndices() {
		System.out.println("***z_updateIndices " + this);
		updatedIndices = true;
		z_updateIndices(0);
		System.out.println("****************************************");
	}
    
    public java.util.List<Item> getItemsClone() {
    	z_updateIndices();
    	return new ArrayList<Item>(items);
    }
	
	@PostLoad
	public void postLoad() {
		System.out.println("!!!postLoad " + this);
		//Comment this out to get rid of the validation errors
		//touchItems();
		System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		updatedIndices = false;
	}
	
	public String toString() {
		return "Container " + this.hashCode() + " id=" + id + ", timestamp=" + timestamp + ", version=" + version;
	}
	
	public String getItemsStr() {
		StringBuilder builder = new StringBuilder();
		for (int i = 0; i<items.size();i++) {
			builder.append("\t" + i + "=> " + items.get(i) + "\n");
        }
		return builder.toString();
	}
	
	static String createTimestamp()
	{
		DateFormat df = new SimpleDateFormat("dd-MMM-yyyy hh:mm:ss.SSS", Locale.UK); 
		df.setTimeZone(TimeZone.getTimeZone("GB"));
		return df.format(new Date());
	}
	
	public void validate(Integer[] order) throws TestException {
		for (int i = 0; i<items.size();i++) {
			Item item = items.get(i);
			int expectedCreationOrder = order[i];
			if(expectedCreationOrder != item.getCreationOrder())
				throw new TestException("Container " + hashCode() + ": Invalid order: expected " + expectedCreationOrder + " got " + item.getCreationOrder());
        }
		
	}
}
